package com.eldos;

import SecureBlackbox.Base.SBEncoding;
import SecureBlackbox.Base.SBUtils;
import SecureBlackbox.Base.TElDCAsyncState;
import SecureBlackbox.Base.TElMemoryStream;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.PDF.TElPDFDocument;
import SecureBlackbox.PDF.TElPDFPublicKeySecurityHandler;
import SecureBlackbox.PDF.TSBPDFPublicKeySignatureType;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;

@WebServlet("/result")
public class ResultServlet extends HttpServlet {
	
	private static final long serialVersionUID = 1L;
	
	private static final String URL = "<a href=\"result?name=%s\">download</a>";

    private Cache cache = Cache.getInstance();

    @Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		
		String sessionID = request.getParameter("sessionid");
		
		if ((sessionID == null) || (request.getContentLength() == 0)) {
			
			response.sendError(400);
			return;
		}
		
		if (! "application/xml".equals(request.getContentType())) {
			
			response.sendError(415);
			return;
		}
		
		InputStream is = request.getInputStream();
		byte[] signature = SBUtils.streamToBuffer(is, 0);
		
		TElMemoryStream result = finishSigning(sessionID, signature);
        cache.put(sessionID, result.getBuffer());

        gotoResult(request, response, "Signed file: " + String.format(URL, sessionID));
	}
	
	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		
		String name = request.getParameter("name");
		
		if (name != null) { // return signed file
		
			byte[] data = cache.get(name);
			if (data == null) {
			
				gotoResult(request, response, "No signed file in cache");
				return;
			}
			
			response.setContentType("application/pdf");
			response.setHeader("Content-Disposition", "filename=sample-" + name + ".pdf");
			
			response.getOutputStream().write(data);
			response.getOutputStream().flush();
			
		} else { // finish signing
			
			String sessionID = request.getParameter("sessionid");
			
			if (sessionID == null) {
				gotoResult(request, response, "Empty session");
				return;
			}
				
			String data = request.getParameter("data");
			if (data != null) {
				byte[] signature = SBEncoding.base64DecodeArray(data);

				TElMemoryStream result = finishSigning(sessionID, signature);
                cache.put(sessionID, result.getBuffer());
			}

            gotoResult(request, response, "Signed file: " + String.format(URL, sessionID));
		}
	}
	
	private void gotoResult(HttpServletRequest request, HttpServletResponse response, String respStr) throws IOException {
		
		request.setAttribute("labelResult", respStr);
		try {
			request.getRequestDispatcher("/result.jsp").forward(request, response);
		} catch (ServletException e) {
			response.sendError(500);
		}
	}

	private TElMemoryStream finishSigning(String sessionID, byte[] signature) {
		TElDCAsyncState state = new TElDCAsyncState();
        TElMemoryStream input = new TElMemoryStream(signature, 0, signature.length);
        state.loadFromStream(input, SBDCXMLEnc.dcxmlEncoding());
        
        TElPDFDocument doc = new TElPDFDocument();
        TElPDFPublicKeySecurityHandler handler = new TElPDFPublicKeySecurityHandler();
        handler.setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);

        byte[] docBin = cache.get(sessionID);
        TElMemoryStream result = new TElMemoryStream(docBin, 0, docBin.length);
        doc.completeAsyncOperation(result, state, handler);
        result.setPosition(0);
        
        return result;
	}
}
