package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.DC.SBDCSec;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPCertRetriever;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.LDAP.SBLDAPCRL;
import SecureBlackbox.LDAP.SBLDAPCertRetriever;
import SecureBlackbox.PDF.*;
import SecureBlackbox.PKIPDF.SBPAdES;
import SecureBlackbox.PKIPDF.TElPDFAdvancedPublicKeySecurityHandler;
import SecureBlackbox.PKIPDF.TSBPAdESSignatureType;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.IOException;

@WebServlet("/select")
public class SelectServlet extends HttpServlet {	
	
	private static final long serialVersionUID = 1L;
	
	static {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");
	
        SBPDF.initialize();
        SBPAdES.initialize();
        SBPDFSecurity.initialize();

        SBHTTPCRL.registerHTTPCRLRetrieverFactory();
        SBLDAPCRL.registerLDAPCRLRetrieverFactory();
        SBHTTPOCSPClient.registerHTTPOCSPClientFactory();
        SBHTTPCertRetriever.registerHTTPCertificateRetrieverFactory();
        SBLDAPCertRetriever.registerLDAPCertificateRetrieverFactory();
	}

    private Cache cache = Cache.getInstance();
	
    @Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		String type = request.getParameter("type");
		String ades = request.getParameter("ades");
		boolean createPAdES = "yes".equals(ades);		
		
		TElMemoryStream output = new TElMemoryStream();

		// prepare
		TElDCAsyncStateCollection States = new TElDCAsyncStateCollection(true);
		try
		{
			presignDocument(request, "sample.pdf", createPAdES, States);
			presignDocument(request, "sample2.pdf", createPAdES, States);
			
			States.saveToStream(output, SBDCXMLEnc.dcxmlEncoding());

			cache.put(request.getSession().getId() + "dochash", output.getBuffer());
		}
		finally
		{
			States.Free();
		}

		// ------------------------------------------ //

        request.setAttribute("paramData", SBEncoding.base64EncodeArray(output.getBuffer(), false));
        request.setAttribute("paramID", request.getSession().getId());
        try {
            if ("activex".equals(type))
                request.getRequestDispatcher("/activex.jsp").forward(request, response);
            else if ("flex".equals(type))
                request.getRequestDispatcher("/flex.jsp").forward(request, response);
            else if ("java".equals(type))
                request.getRequestDispatcher("/java.jsp").forward(request, response);
            else
                response.sendError(400);
        } catch (ServletException e) {
            response.sendError(500);
        }
	}
  
	  private void presignDocument(HttpServletRequest request, String filename, Boolean createPAdES, TElDCAsyncStateCollection states)
	  {
		  // temporary copy of the document
		  byte[] docBin = Utils.getPDFFile(this, filename);
		  TElMemoryStream stream = new TElMemoryStream(docBin, 0, docBin.length);
	    
		  TElDCAsyncState state;

		  TElPDFDocument Document = new TElPDFDocument();
		  TElPDFSecurityHandler handler;
		  TElPDFSignature signature;
		  try
		  {     
			  Document.open(stream);
	        
			  if (createPAdES) {
				  handler = new TElPDFAdvancedPublicKeySecurityHandler();
	        	
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setPAdESSignatureType(TSBPAdESSignatureType.pastEnhanced);
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
				  ((TElPDFAdvancedPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
			  } else {
				  handler = new TElPDFPublicKeySecurityHandler();
	        	
				  ((TElPDFPublicKeySecurityHandler)handler).setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);
				  ((TElPDFPublicKeySecurityHandler)handler).setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
				  ((TElPDFPublicKeySecurityHandler)handler).setCustomName("Adobe.PPKMS");
			  }
	        
			  signature = Document.getSignatureEntry(Document.addSignature());
			  signature.setSignatureType(SBPDF.stDocument);
			  signature.setInvisible(true);
			  signature.setHandler(handler);
			  signature.setSigningTime(SBUtils.utcNow());
			  //signature.setExtraSpace(500 * 1024); // if "EElPDFError: No enough space for signature" occurs    

                          TElDCParameters pars = new TElDCParameters();

                          pars.Parameters.Add(SBStrUtils.StrToUTF8("DataURL"), SBStrUtils.StrToUTF8("/dcweb/result"));
                          pars.Parameters.Add(SBStrUtils.StrToUTF8("GoURL"), SBStrUtils.StrToUTF8("/dcweb/result"));
                          pars.Parameters.Add(SBStrUtils.StrToUTF8("SessionID"), SBStrUtils.StrToUTF8(request.getSession().getId()));
                          pars.Parameters.Add(SBStrUtils.StrToUTF8("PKCS11Registry"), SBStrUtils.StrToUTF8("\\Software\\EldoS\\SecureBlackbox\\DC"));

                          if (createAdES)
                                  pars.AsyncSignMethod = TSBDCAsyncSignMethod.asmPKCS7;
                          else
                                  pars.AsyncSignMethod = TSBDCAsyncSignMethod.asmPKCS1;

                          state = Document.initiateAsyncOperation(pars);
			  
			  states.add(state, false);

			  cache.put(request.getSession().getId() + "." + Integer.toString(states.getCount()), stream.getBuffer());
		  }
		  finally
		  {
			  handler.Free();
			  Document.Free();
			  stream.Free();
		  }        
	}
}
