package com.eldos;

import java.net.URLDecoder;
import java.util.Map;
import java.util.LinkedHashMap;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.tomcat.util.http.fileupload.IOUtils;

import SecureBlackbox.Base.SBEncoding;

@WebServlet("/AppletJnlp")
public class JnlpServlet extends HttpServlet {
	
	private static final long serialVersionUID = 1L;
	
	private Cache cache = Cache.getInstance();
       
    public JnlpServlet() {
        super();
    }

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		// Get session id from the query string in order to find the signing 
		// information in the cache.
		
		Map<String, String> queryPairs = splitQuery(request.getQueryString());
		String sessionId = queryPairs.get("SessionId");
		if (sessionId == null) {
			response.sendError(400);
			return;
		}
		
		// Obtain the document hash from the cache and convert in to base64 string.
		
		byte[] docHash = cache.get(sessionId + "dochash");
		if (docHash == null) {
			response.sendError(400);
			return;
		}
		String sDocHash = SBEncoding.base64EncodeArray(docHash, false);
		
		// Compose JNLP-file with the required for the signing information.
		
		StringBuffer codebase = request.getRequestURL();
		char ch = codebase.charAt(codebase.length()-1);
		if (ch == '/' || ch == '\\')
			codebase.setLength(codebase.length()-1);
		for (int i = codebase.length()-1; i > 0; i--) {
			
			ch = codebase.charAt(i);
			if (ch == '/' || ch == '\\') {
				codebase.setLength(i);
				break;
			}
		}
		
		byte[] fileData = Utils.getJNLPFile(this, codebase.toString(), sessionId, sDocHash);
		
		// Return the JNLP-file. 

		ByteArrayInputStream in = new ByteArrayInputStream(fileData); 
		
		response.setContentType("application/x-java-jnlp-file");
		response.setHeader("content-disposition", "attachment; filename=\"DCServer_" + sessionId + ".jnlp\"");
		
		IOUtils.copy(in, response.getOutputStream());
	}
	
	private static Map<String, String> splitQuery(String query) throws UnsupportedEncodingException {
		
	    Map<String, String> queryPairs = new LinkedHashMap<String, String>();
	    
	    if (query == null)
	    	return queryPairs;
	    
	    String[] pairs = query.split("&");
	    for (String pair : pairs) {
	        int idx = pair.indexOf("=");
	        queryPairs.put(URLDecoder.decode(pair.substring(0, idx), "UTF-8"), URLDecoder.decode(pair.substring(idx + 1), "UTF-8"));
	    }
	    return queryPairs;
	}
}
