package com.eldos;

import SecureBlackbox.Base.*;
import SecureBlackbox.DC.SBDCXMLEnc;
import SecureBlackbox.DC.SBDCSec;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPCertRetriever;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.LDAP.SBLDAPCRL;
import SecureBlackbox.LDAP.SBLDAPCertRetriever;
import SecureBlackbox.XML.SBXMLDefs;
import SecureBlackbox.XML.TElXMLDOMDocument;
import SecureBlackbox.XMLSecurity.*;

import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@WebServlet("/select")
public class SelectServlet extends HttpServlet {	
	
	private static final long serialVersionUID = 1L;
	
	static {
		
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");

        SBHTTPCRL.registerHTTPCRLRetrieverFactory();
        SBLDAPCRL.registerLDAPCRLRetrieverFactory();
        SBHTTPOCSPClient.registerHTTPOCSPClientFactory();
        SBHTTPCertRetriever.registerHTTPCertificateRetrieverFactory();
        SBLDAPCertRetriever.registerLDAPCertificateRetrieverFactory();
	}
	
    private Cache cache = Cache.getInstance();
	
    @Override
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
    	
		String type = request.getParameter("type");
		String ades = request.getParameter("ades");
		boolean createAdES = "yes".equals(ades);
    			
		// temporary copy of the document
		
		byte[] docBin = Utils.getXMLFile(this);
		TElMemoryStream doc = new TElMemoryStream(docBin, 0, docBin.length);
		TElMemoryStream output = new TElMemoryStream();
		
		// prepare
		
		TElDCAsyncState state;

        TElXMLDOMDocument xml = new TElXMLDOMDocument();
        xml.loadFromStream(doc);

        TElXMLSigner signer = new TElXMLSigner();
        signer.setSignatureType(SBXMLSec.xstEnveloped);
        signer.setSignatureMethodType(SBXMLSec.xmtSig);
        signer.setSignatureMethod(SBXMLSec.xsmRSA_SHA1);
        signer.setIncludeKey(true);

        if (createAdES) {
            TElXAdESSigner xadesSigner = new TElXAdESSigner();
            signer.setXAdESProcessor(xadesSigner);

            xadesSigner.setXAdESVersion(SBXMLAdES.XAdES_v1_3_2);
            xadesSigner.setSigningTime(SBUtils.utcNow());

            xadesSigner.generate();
            xadesSigner.getQualifyingProperties().setXAdESPrefix("xades");
        }

        TElXMLReference ref = new TElXMLReference();
        ref.setDigestMethod(SBXMLSec.xdmSHA1);
        ref.setURINode(xml.getDocumentElement());
        ref.setURI("");
        ref.getTransformChain().add(new TElXMLEnvelopedSignatureTransform());
        signer.getReferences().add(ref);

        signer.updateReferencesDigest();
        signer.generateSignatureAsync();

        TElDCParameters pars = new TElDCParameters();

        pars.Parameters.Add(SBStrUtils.StrToUTF8("DataURL"), SBStrUtils.StrToUTF8("/dcweb/result"));
        pars.Parameters.Add(SBStrUtils.StrToUTF8("GoURL"), SBStrUtils.StrToUTF8("/dcweb/result"));
        pars.Parameters.Add(SBStrUtils.StrToUTF8("SessionID"), SBStrUtils.StrToUTF8(request.getSession().getId()));
        pars.Parameters.Add(SBStrUtils.StrToUTF8("PKCS11Registry"), SBStrUtils.StrToUTF8("\\Software\\EldoS\\SecureBlackbox\\DC"));

        TSBObject var = new TSBObject();
        var.value = xml.getDocumentElement();

        state = signer.initiateAsyncSign(pars, var);
        state.saveToStream(output, SBDCXMLEnc.dcxmlEncoding());

        doc.setLength(0);
        xml.saveToStream(doc, SBXMLDefs.xcmNone, "");

        cache.put(request.getSession().getId(), doc.getBuffer());
        cache.put(request.getSession().getId() + "dochash", output.getBuffer());
        
		// ------------------------------------------ //

        request.setAttribute("paramData", SBEncoding.base64EncodeArray(output.getBuffer(), false));
        request.setAttribute("paramID", request.getSession().getId());
        try {
            if ("activex".equals(type))
            	request.getRequestDispatcher("/activex.jsp").forward(request, response);
            else if ("flex".equals(type))
            	request.getRequestDispatcher("/flex.jsp").forward(request, response);
            else if ("java".equals(type))
            	request.getRequestDispatcher("/java.jsp").forward(request, response);
            else
            	response.sendError(400);
        } catch (ServletException e) {
        	response.sendError(500);
        }
	}
}
