

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.SSHClient.*;
import SecureBlackbox.SSHCommon.*;

public class Main {

    static TElSimpleSSHClient client = null;

    static String parCommand;
    static String parUsername;
    static String parPassword;
    static String parAddress;
    static String parPrivateKey;
    static String parPrivateKeyPass = "";
    static int parPort;
	
	public static void main(String[] args) {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
		
        parPort = 22;

        if (!ParseCommandLine(args))
            return;

        client = new TElSimpleSSHClient();
        client.setAddress(parAddress);
        client.setPort(parPort);

        client.setUsername(parUsername);
        client.setPassword(parPassword);
        
        if (parPrivateKey != null && ( ! "".equals(parPrivateKey))) {
        	TElSSHKey key = new TElSSHKey();
        	int err = key.loadPrivateKey(parPrivateKey, parPrivateKeyPass);
        	if (err != 0) {
        		System.out.println("Failed to load private key: " + err);
        		return;
        	}
        	 
        	TElSSHMemoryKeyStorage storage = new TElSSHMemoryKeyStorage();
        	storage.add(key);
        	
        	client.setAuthenticationTypes(client.getAuthenticationTypes() | SBSSHConstants.SSH_AUTH_TYPE_PUBLICKEY);
        	client.setKeyStorage(storage);
        }

        client.setOnKeyValidate(new TSSHKeyValidateEvent(OnKeyValidate));

        try
        {
            byte[] resp = client.executeCommand(parCommand);
            String output = new String(resp);
            System.out.print(output);
        }
        catch (Exception e)
        {
            System.out.println("Connection failed due to exception: " + e.getMessage());
            System.out.println("If you have ensured that all connection parameters are correct and you still can't connect,");
            System.out.println("please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php");
            System.out.println("Remember to provide details about the error that happened.");
            if (client.getServerSoftwareName().length() > 0)
            {
                System.out.println("Server software identified itself as: " + client.getServerSoftwareName());
            }
        }
	}

	static TSSHKeyValidateEvent.Callback OnKeyValidate = new TSSHKeyValidateEvent.Callback() {
		
		public void tsshKeyValidateEventCallback(TObject arg0, TElSSHKey arg1, TSBBoolean accept) {
	        System.out.println("Server key received");
			accept.value = true; // NEVER do this. You MUST check the key validity somehow
		}
	};
	
    static boolean ParseCommandLine(String[] args)
    {
        int i = 0;
        String arg;

        if (args.length == 0)
        {
            System.out.println("usage:");
            System.out.println("ConsoleSSH -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> -key <key path> -keypass <key password> -command <command to execute remotely>");
            return false;
        }
        else
        {
            while (i < args.length)
            {
                arg = args[i];
                if (arg.compareTo("-command") == 0)
                {
                    i++;
                    parCommand = args[i];
                }
                else if ((arg.compareTo("-addr") == 0) ||
                  (arg.compareTo("-address") == 0))
                {
                    i++;
                    parAddress = args[i];
                }
                else if (arg.compareTo("-port") == 0)
                {
                    i++;
                    try
                    {
                        parPort = Integer.parseInt(args[i]);
                    }
                    catch (Exception ex)
                    {
                        parPort = 22;
                    }
                }
                else if ((arg.compareTo("-user") == 0) ||
                    (arg.compareTo("-username") == 0))
                {
                    i++;
                    parUsername = args[i];
                }
                else if ((arg.compareTo("-pass") == 0) ||
                    (arg.compareTo("-password") == 0))
                {
                    i++;
                    parPassword = args[i];
                }
                else if (arg.compareTo("-key") == 0)
                {
                	i++;
                	parPrivateKey = args[i];
                }
                else if (arg.compareTo("-keypass") == 0)
                {
                	i++;
                	parPrivateKeyPass = args[i];
                }
                i = i + 1;
            }
            return (parCommand != "");
        }
    }
}
